
From: Dominik Brodowski <linux@dominikbrodowski.net>

- properly baild out in set_cis_map if call to socket driver's set_mem_map
  failed
- don't abort do_mem_probe cycle if one entry failed (!CONFIG_PCMCIA_PROBE)
- don't do iomem probing in chunks larger than 0x800000 (1 << 23) as
  yenta_socket and vrc4173_cardu.c fail to set_mem_map for windows equal to 
  or larger than (1 << 24).

Signed-off-by: Dominik Brodowski <linux@dominikbrodowski.net>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/drivers/pcmcia/cistpl.c         |    9 ++++++++-
 25-akpm/drivers/pcmcia/rsrc_nonstatic.c |    6 ++++--
 2 files changed, 12 insertions(+), 3 deletions(-)

diff -puN drivers/pcmcia/cistpl.c~pcmcia-resource-handling-fixes drivers/pcmcia/cistpl.c
--- 25/drivers/pcmcia/cistpl.c~pcmcia-resource-handling-fixes	2005-03-31 02:01:19.000000000 -0800
+++ 25-akpm/drivers/pcmcia/cistpl.c	2005-03-31 02:01:19.000000000 -0800
@@ -89,6 +89,8 @@ static void __iomem *
 set_cis_map(struct pcmcia_socket *s, unsigned int card_offset, unsigned int flags)
 {
     pccard_mem_map *mem = &s->cis_mem;
+    int ret;
+
     if (!(s->features & SS_CAP_STATIC_MAP) && mem->res == NULL) {
 	mem->res = pcmcia_find_mem_region(0, s->map_size, s->map_size, 0, s);
 	if (mem->res == NULL) {
@@ -99,7 +101,12 @@ set_cis_map(struct pcmcia_socket *s, uns
     }
     mem->card_start = card_offset;
     mem->flags = flags;
-    s->ops->set_mem_map(s, mem);
+    ret = s->ops->set_mem_map(s, mem);
+    if (ret) {
+	iounmap(s->cis_virt);
+	return NULL;
+    }
+
     if (s->features & SS_CAP_STATIC_MAP) {
 	if (s->cis_virt)
 	    iounmap(s->cis_virt);
diff -puN drivers/pcmcia/rsrc_nonstatic.c~pcmcia-resource-handling-fixes drivers/pcmcia/rsrc_nonstatic.c
--- 25/drivers/pcmcia/rsrc_nonstatic.c~pcmcia-resource-handling-fixes	2005-03-31 02:01:19.000000000 -0800
+++ 25-akpm/drivers/pcmcia/rsrc_nonstatic.c	2005-03-31 02:01:19.000000000 -0800
@@ -372,6 +372,9 @@ static int do_mem_probe(u_long base, u_l
 	   base, base+num-1);
     bad = fail = 0;
     step = (num < 0x20000) ? 0x2000 : ((num>>4) & ~0x1fff);
+    /* don't allow too large steps */
+    if (step > 0x800000)
+	step = 0x800000;
     /* cis_readable wants to map 2x map_size */
     if (step < 2 * s->map_size)
 	step = 2 * s->map_size;
@@ -465,8 +468,7 @@ static void validate_mem(struct pcmcia_s
 
 	for (m = s_data->mem_db.next; m != &s_data->mem_db; m = mm.next) {
 		mm = *m;
-		if (do_mem_probe(mm.base, mm.num, s))
-			break;
+		do_mem_probe(mm.base, mm.num, s);
 	}
 }
 
_
